import Ember from 'ember';
const {Component, computed, observer} = Ember;


export default Component.extend({
  classNames: ['modal'],
  classNameBindings: ['fade:fade'],
  hideClass: 'hide.bs.modal',
  hiddenClass: 'hidden.bs.modal',
  keyboard: true,
  backdrop: true,
  closeAfterSuccess: false,
  closeAfterFail: false,
  staticBackdrop: false,
  doAfterClose: 'doAfterClose',
  doSave: 'doSave',
  doClose: 'doClose',
  _isSubmitting: false,
  _isClosing: false,
  _isReadyToClose: false,
  showOnInit: true,
  fade: true,
  disabled: computed.or('_isSubmitting', '_isClosing'),

  backdropSetting: computed('backdrop', 'staticBackdrop', function() {
    var res = this.get('backdrop');
    return (res && this.get('staticBackdrop')) ? 'static' : res;
  }),

  modalOptions: computed('backdropSetting', 'keyboard', 
    function() {
      return {
          backdrop: this.get('backdropSetting'),
          keyboard: this.get('keyboard')
      };
    }
  ),

  actions: {

    submit(meta) {
      var self = this,
        defer = Ember.RSVP.defer();

      self.set('_isSubmitting', true);

      defer.promise.then(
        function() {
          if(self.get('closeAfterSuccess')) {
            self.set('_isReadyToClose', true);
            self.$().modal('hide');
          } else {
            self.set('_isSubmitting', false);
          }
        }, 
        function() {
          if(self.get('closeAfterFail')) {
            self.set('_isReadyToClose', true);
            self.$().modal('hide');
          } else {
            self.set('_isSubmitting', false);
          }
        }
      );
      self.sendAction('doSave', defer, meta);
    }
  },

  isVisibleChanged: observer('isVisible', function() {
    if(this.get('isVisible')) {
      this.renderModal();
    } else {
      this.destroyModal();
    }
  }),

  renderModal: function() {
    var self = this;
    
    self.$().modal(self.get('modalOptions'));

    self.$().modal().on(self.get('hideClass'), function(e) {
        Ember.run(function() {
          e.stopImmediatePropagation();
          self.set('_isClosing', true);
          if(self.get('_isReadyToClose')) { return; }
          e.preventDefault();
          var defer = Ember.RSVP.defer();
          defer.promise.then(
            function() {
              self.set('_isReadyToClose', true);
              self.$().modal('hide');
            }, 
            function() {
              self.setProperties({
                _isClosing: false,
                _isReadyToClose: false
              });
            }
          );
          self.sendAction('doClose', defer);
        });

    });

    self.$().modal().on(self.get('hiddenClass'), 
      function() {
        Ember.run(function() {
          self.destroyModal();
          self.setProperties({
              isVisible: false,
              _isReadyToClose: false,
              _isClosing: false,
              _isSubmitting: false
          });
          self.sendAction(self.get('doAfterClose'));
        });
      }
    );

  },

  destroyModal: function() {
    this.$().off(this.get('hiddenClass'));
  },
  
  setupBootstrapModal: function() {
    this._super();
    Ember.run.scheduleOnce('afterRender', this, this.afterRenderEvent);
  }.on('didInsertElement'),

  afterRenderEvent: function() {
    if(this.get('showOnInit')) {
      this.set('isVisible', true);
    }
  },

  teardownBootstrapModal: function() {
    this.destroyModal();

  }.on('willDestroyElement')

});
